#include "kernel.h"
#include "pci.h"

/*===========================================================================*
 *                              scan_bus                                     *
 *===========================================================================*/
void scan_bus(pcidev)
struct pci_device *pcidev;
{
  unsigned int      devfn, l, bus, buses;
  unsigned char     hdr_type = 0;
  unsigned short    vendor, device;
  unsigned int      membase, ioaddr, romaddr;
  unsigned char     irq;
  unsigned char     class, subclass;
  int               i, reg;
  unsigned int      pci_ioaddr = 0;
  struct rom_info   rom;

/*  rom = *(struct rom_info *)ROM_INFO_LOCATION; */

  /* scan PCI BIOS info. */
  buses=1;
  for (bus = 0; bus < buses; ++bus) {
    for (devfn = 0; devfn < 0xff; ++devfn) {
      if (PCI_FUNC (devfn) == 0)
        pcibios_read_config_byte(bus, devfn, PCI_HEADER_TYPE, &hdr_type);
      else if (!(hdr_type & 0x80))    /* not a multi-function device */
        continue;
      pcibios_read_config_dword(bus, devfn, PCI_VENDOR_ID, &l);
      /* some broken boards return 0 if a slot is empty: */
      if (l == 0xffffffff || l == 0x00000000) {
        hdr_type = 0;
        continue;
      }
      vendor = l & 0xffff;
      device = (l >> 16) & 0xffff;
      
      /* check for pci-pci bridge devices!! - more buses when found */
      pcibios_read_config_byte(bus, devfn, PCI_CLASS_CODE, &class);
      pcibios_read_config_byte(bus, devfn, PCI_SUBCLASS_CODE, &subclass);
      if (class == 0x06 && subclass == 0x04)
        buses++;

      for (i = 0; pcidev[i].vendor != 0; i++) {
        if (vendor != pcidev[i].vendor
            || device != pcidev[i].dev_id || pcidev[i].suffix != 0)
          continue;
        pcidev[i].devfn = devfn;
        pcidev[i].bus = bus;
        for (reg = PCI_BASE_ADDRESS_0; reg <= PCI_BASE_ADDRESS_5; reg += 4) {
          pcibios_read_config_dword(bus, devfn, reg, &ioaddr);

          if ((ioaddr & PCI_BASE_ADDRESS_IO_MASK) == 0 
              || (ioaddr & PCI_BASE_ADDRESS_SPACE_IO) == 0)
            continue;
          /* Strip the I/O address out of the returned value */
          ioaddr &= PCI_BASE_ADDRESS_IO_MASK;
          /* Get the memory base address */
          pcibios_read_config_dword(bus, devfn,
                                    PCI_BASE_ADDRESS_1, &membase);
          /* KK: Get the IRQ number */
          pcibios_read_config_byte(bus, devfn, PCI_INTERRUPT_PIN, &irq);
          if (irq)
            pcibios_read_config_byte(bus, devfn, PCI_INTERRUPT_LINE, &irq);
          /* Get the ROM base address */
          pcibios_read_config_dword(bus, devfn, PCI_ROM_ADDRESS, &romaddr);
          romaddr >>= 10;
          /* Take the first one or the one that matches in boot ROM address */
          if (pci_ioaddr == 0 
              || romaddr == ((unsigned long) rom.rom_segment << 4)) {
            pcidev[i].membase = membase;
            pcidev[i].ioaddr = ioaddr;
            pcidev[i].irq = irq;
	    pcidev[i].suffix = 1;
          }
        }
	if (pcidev[i].suffix != 0)
	  break;
      }

    }
  }
}

/*===========================================================================*
 *                            pcibios_*_config_*                             *
 *===========================================================================*/
int pcibios_read_config_byte(unsigned int bus, unsigned int device_fn,
				    unsigned int where, unsigned char *value)
{
    out_long(0xCF8, CONFIG_CMD(bus,device_fn,where));
    *value = in_byte(0xCFC + (where&3));
    return PCIBIOS_SUCCESSFUL;
}

int pcibios_read_config_word (unsigned int bus, unsigned int device_fn, 
				     unsigned int where, unsigned short *value)
{
    out_long(0xCF8, CONFIG_CMD(bus,device_fn,where));
    *value = in_word(0xCFC + (where&2));
    return PCIBIOS_SUCCESSFUL;
}

int pcibios_read_config_dword (unsigned int bus, unsigned int device_fn,
				      unsigned int where, unsigned int *value)
{
    out_long(0xCF8, CONFIG_CMD(bus,device_fn,where));
    *value = in_long(0xCFC);
    return PCIBIOS_SUCCESSFUL;
}

int pcibios_write_config_word (unsigned int bus, unsigned int device_fn,
				      unsigned int where, unsigned short value)
{
    out_long(0xCF8, CONFIG_CMD(bus,device_fn,where));
    out_word(0xCFC + (where&2), value);
    return PCIBIOS_SUCCESSFUL;
}

#if 0
/*===========================================================================*
 *                              in_long                                      *
 *===========================================================================*/
unsigned int in_long(port_t port)
{
  unsigned int data;
  __asm__ __volatile__("inl %%dx,%0" : "=a" (data) : "d" (port));
  return data;
}

/*===========================================================================*
 *                              out_long                                     *
 *===========================================================================*/
void out_long(port_t port, unsigned int value)
{
  unsigned int data = value;
  __asm__ __volatile__("outl %0,%%dx" : : "a" (data), "d" (port));
}
#endif
