/*
 * pty.h
 *
 * created from pty.c in order to support select() in tty.c
 *
 * GH  23/11/98
 */

/* PTY bookkeeping structure, one per pty/tty pair. */
typedef struct pty {
  tty_t		*tty;		/* associated TTY structure */
  char		state;		/* flags: busy, closed, ... */

  /* Read call on /dev/ptypX. */
  char		rdrepcode;	/* reply code, TASK_REPLY or REVIVE */
  char		rdcaller;	/* process making the call (usually FS) */
  char		rdproc;		/* process that wants to read from the pty */
  vir_bytes	rdvir;		/* virtual address in readers address space */
  int		rdleft;		/* # bytes yet to be read */
  int		rdcum;		/* # bytes written so far */

  /* Write call to /dev/ptypX. */
  char		wrrepcode;	/* reply code, TASK_REPLY or REVIVE */
  char		wrcaller;	/* process making the call (usually FS) */
  char		wrproc;		/* process that wants to write to the pty */
  vir_bytes	wrvir;		/* virtual address in writers address space */
  int		wrleft;		/* # bytes yet to be written */
  int		wrcum;		/* # bytes written so far */

  /* Tnet 4.0 */
  int		pty_line;	/* line number of this pty */
  int		pty_sel_read;	/* proc_nr of select call or zero */
  int		pty_sel_write;	/* proc_nr of select call or zero */

  /* Output buffer. */
  int		ocount;		/* # characters in the buffer */
  char		*ohead, *otail;	/* head and tail of the circular buffer */
  char		obuf[128];	/* buffer for bytes going to the pty reader */

} pty_t;

#define PTY_ACTIVE	0x01	/* pty is open/active */
#define TTY_CLOSED	0x02	/* tty side has closed down */
#define PTY_CLOSED	0x04	/* pty side has closed down */

#if (NR_PTYS > 0)
EXTERN pty_t pty_table[NR_PTYS];       /* PTY bookkeeping */
#endif /* NR_PTYS > 0 */
